package com.sagacloud.route.processors.PeriodTask;
/*
 * Author: Jxing
 * Create Time: 2018/7/13
 */

import com.alibaba.fastjson.JSONObject;
import com.sagacloud.pojos.DPSelectPropertyResult;
import com.sagacloud.pojos.ProjectStatistics;
import com.sagacloud.pojos.VenderStatistics;
import com.sagacloud.pojos.Warranty;
import com.sagacloud.route.InitEnvRoute;
import com.sagacloud.route.processors.Insurance.GetPropertyUnderWarrantyProcessor;
import com.sagacloud.utils.Const;
import com.sagacloud.utils.VendersUtil;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;

import java.util.*;

public class StatisticsProcessor implements Processor {
    public static final Set<String> manufacturerSign = VendersUtil.createSet("Manufacturer", "Brand", "Specification", "DPManufacturerID", "DPSpecificationID");
    public static final Set<String> supplierSign = VendersUtil.createSet("Supplier", "DPSupplierID");
    public static final Set<String> insuranceSign = VendersUtil.createSet("Insurer", "InsuranceNum", "DPInsurerID");
    public static final Set<String> maintainanceSign = VendersUtil.createSet("Maintainer", "DPMaintainerID");
    @Override
    public void process(Exchange exchange) throws Exception {
        String projectId = exchange.getIn().getBody(String.class);
        //System.out.println("这里正在统计的是Project：" + projectId);
        DPSelectPropertyResult propertyData = (DPSelectPropertyResult) exchange.getProperty("propertyData" + exchange.getExchangeId());
        DPSelectPropertyResult facilityData = (DPSelectPropertyResult) exchange.getProperty("facilityData" + exchange.getExchangeId());
        Map<String, Map<String, String>> infoValueMap = (Map<String, Map<String, String>>) exchange.getProperty("map");  // 信息点Code --> (厂商库在该信息点的值value --> vender DPID)
        List<Map<String, Object>> propertyList = propertyData.getContent();
        List<Map<String, Object>> facilityList = facilityData.getContent();
        Map<String, VenderStatistics> venderStatisticsMap = new HashMap<>();
        Map<String, Set<String>> contracts = new HashMap<>(); // 供应商统计供应合同数目使用
        Map<String, Set<String>> warranties = new HashMap<>(); // 保险商统计供应合同数目使用
        Map<String, Warranty> newWarranty = new HashMap<>(); // 记录数据平台新增保单, 增加到厂商库
        exchange.setProperty("newWarranty", newWarranty);
        for(int i = 0; i < propertyList.size(); ++i){
            Map<String, Object> singleProperty = propertyList.get(i);
            matchCountManufacturer(singleProperty, infoValueMap, manufacturerSign, venderStatisticsMap);
            matchCountSupplier(singleProperty, infoValueMap, supplierSign, venderStatisticsMap, contracts);
            matchCountInsurer(singleProperty, infoValueMap, insuranceSign, venderStatisticsMap, warranties, newWarranty, projectId);
            //
        }
        for(int i = 0; i < facilityList.size(); ++i){
            Map<String, Object> singleFacility = facilityList.get(i);
            matchCountMaintainance(singleFacility, infoValueMap, maintainanceSign, venderStatisticsMap);
        }
        addContractCount(venderStatisticsMap, contracts);
        addContractCount(venderStatisticsMap, warranties);
        ProjectStatistics ps = new ProjectStatistics();
        if(venderStatisticsMap.size() == 0){
            ps.setCriterias(new ArrayList<>());
        }else {
            ps.setCriterias(new ArrayList<>(venderStatisticsMap.values()));
        }
        ps.setProjectId(projectId);
        exchange.getIn().setBody(JSONObject.toJSONString(ps));
        //System.out.println("项目:" + projectId + JSONObject.toJSONString(ps));
        exchange.getIn().setHeader(Exchange.HTTP_URI, String.join("", InitEnvRoute.venders, "/auxiliary/property/create_by_project?method=POST"));
    }

    private void addContractCount(Map<String, VenderStatistics> venderStatisticsMap, Map<String, Set<String>> contracts) {
        for(String venderId : contracts.keySet()){
            updateCount(venderStatisticsMap, venderId, 0, contracts.get(venderId).size(), 0);
        }
    }

    private void matchCountMaintainance(Map<String, Object> singleProperty, Map<String, Map<String, String>> infoValueMap, Set<String> maintainanceSign, Map<String, VenderStatistics> venderStatisticsMap) {
        try {
            Map<String, Object> propertyInfos = (Map<String, Object>) singleProperty.get("infos");
            for(String idInfoCode : maintainanceSign){
                Map<String, String> valueMap = infoValueMap.get(idInfoCode);
                if(propertyInfos.get(idInfoCode) == null || valueMap == null)
                    continue;
                String infoValue = propertyInfos.get(idInfoCode).toString();   //当前信息点的值
                if(valueMap.containsKey(infoValue)){
                    // 匹配由此保险商保的资产
                    String venderId = valueMap.get(infoValue);
                    if(propertyInfos.get(Const.MTN_DEADLINE) != null) {
                        String deadline = propertyInfos.get(Const.MTN_DEADLINE).toString();
                        try{  // 比较deadline 是否是快过期的
                            Calendar c = Calendar.getInstance();
                            c.add(Calendar.MONTH, 6);
                            Date now = new Date();
                            Date upcomingDate = c.getTime();
                            Date expireDate = GetPropertyUnderWarrantyProcessor.sdf.parse(deadline);
                            if(expireDate.compareTo(upcomingDate) >= 0){
                                updateCount(venderStatisticsMap, venderId, 1, 0, 0);
                            }else if(expireDate.compareTo(now) >= 0){
                                updateCount(venderStatisticsMap, venderId, 0, 0, 1);
                            }
                        }catch (Exception ignored){
                            updateCount(venderStatisticsMap, venderId, 0, 0, 1);
                        }
                        return;
                    }else    // 如果没有填时间, 则统计在即将超出维保时间的资产中
                    {
                        updateCount(venderStatisticsMap, venderId, 0, 0, 1);
                        return;
                    }
                }
            }
        }catch (Exception ignored){}
    }

    // InsuranceNum 在infoValueMap里面保存的已经是所有的在保保单号, 因此匹配到资产之后必须验证资产保单号是否属于在保保单
    private void matchCountInsurer(Map<String, Object> singleProperty, Map<String, Map<String, String>> infoValueMap, Set<String> insuranceSign, Map<String, VenderStatistics> venderStatisticsMap, Map<String, Set<String>> warranties, Map<String, Warranty> newWarranty, String projectId) {
        try {
            Map<String, Object> propertyInfos = (Map<String, Object>) singleProperty.get("infos");
            for(String idInfoCode : insuranceSign){
                Map<String, String> valueMap = infoValueMap.get(idInfoCode);
                if(propertyInfos.get(idInfoCode) == null || valueMap == null)
                    continue;
                String infoValue = propertyInfos.get(idInfoCode).toString();   //资产当前信息点的值
                if(valueMap.containsKey(infoValue)){
                    String venderId = valueMap.get(infoValue);
                    // 匹配到由此保险商保的资产
                    if(propertyInfos.get(Const.INFO_INSURANCENUM) != null) {
                        String insuranceNum = propertyInfos.get(Const.INFO_INSURANCENUM).toString();
                        Map<String, String> insuranceNumMap = infoValueMap.get(Const.INFO_INSURANCENUM);
                        // 匹配 保单号(统计被保资产个数)
                        if (insuranceNumMap.containsKey(insuranceNum)) {
                            venderId = insuranceNumMap.get(insuranceNum);
                            Set<String> insuranceSet = null;
                            if (warranties.get(venderId) == null) {
                                insuranceSet = new HashSet<>();
                                warranties.put(venderId, insuranceSet);
                            } else
                                insuranceSet = warranties.get(venderId);
                            if (!insuranceSet.contains(insuranceNum))
                                insuranceSet.add(insuranceNum);
                            updateCount(venderStatisticsMap, venderId, 1, 0, 0);
                        }else{
                            Map<String, String> allInsuranceNum =  infoValueMap.get("AllInsuranceNum");
                            if(!allInsuranceNum.containsKey(insuranceNum)){
                                Warranty warranty = new Warranty();
                                warranty.setProjectId(projectId);
                                warranty.setInsuranceNo(insuranceNum);
                                warranty.setVenderId(venderId);
                                if(propertyInfos.get("InsuranceFile") != null)
                                    warranty.setContractFile(propertyInfos.get("InsuranceFile").toString());
                                if(!newWarranty.containsKey(insuranceNum)){
                                    newWarranty.put(insuranceNum, warranty);
                                }
                            }
                        }
                        return;
                    }
                }
            }
        }catch (Exception ignored){}
    }

    private void matchCountSupplier(Map<String, Object> singleProperty, Map<String, Map<String, String>> infoValueMap, Set<String> supplierSign, Map<String, VenderStatistics> venderStatisticsMap, Map<String, Set<String>> contracts) {
        try {
            Map<String, Object> propertyInfos = (Map<String, Object>) singleProperty.get("infos");
            for(String idInfoCode : supplierSign){
                Map<String, String> valueMap = infoValueMap.get(idInfoCode);
                if(propertyInfos.get(idInfoCode) == null || valueMap == null)
                    continue;
                String infoValue = propertyInfos.get(idInfoCode).toString();
                if(valueMap.containsKey(infoValue)){
                    // 匹配由此供应商提供的资产
                    String venderId = valueMap.get(infoValue);
                    if(propertyInfos.get(Const.INFO_SUPPLIERCONTRACTID) != null){
                        String contractNo = propertyInfos.get(Const.INFO_SUPPLIERCONTRACTID).toString();
                        Set<String> contractSet = null;
                        if(contracts.get(venderId) == null){
                            contractSet = new HashSet<>();
                            contracts.put(venderId, contractSet);
                        }else
                            contractSet = contracts.get(venderId);
                        if(!contractSet.contains(contractNo))
                            contractSet.add(contractNo);
                    }
                    updateCount(venderStatisticsMap, venderId, 1, 0, 0);
                    return;
                }
            }
        }catch (Exception ignored){}
    }

    private void matchCountManufacturer(Map<String, Object> singleProperty, Map<String, Map<String, String>> infoValueMap, Set<String> venderSign, Map<String, VenderStatistics> venderStatisticsMap) {
        try {
            Map<String, Object> propertyInfos = (Map<String, Object>) singleProperty.get("infos");
            for(String idInfoCode : venderSign){
                Map<String, String> valueMap = infoValueMap.get(idInfoCode);
                if(propertyInfos.get(idInfoCode) == null || valueMap == null)
                    continue;
                String infoValue = propertyInfos.get(idInfoCode).toString();
                if(valueMap.containsKey(infoValue)){
                    // 匹配成功, 由此生产商生产
                    String venderId = valueMap.get(infoValue);
                    updateCount(venderStatisticsMap, venderId, 1, 0, 0);
                    return;
                }
            }
        }catch (Exception ignored){}
    }
    private void updateCount(Map<String, VenderStatistics> venderStatisticsMap, String venderId, int totalCount, int contractCount, int nearlyExpireCount){
        VenderStatistics statistics = null;
        if(venderStatisticsMap.get(venderId) == null){
            statistics = new VenderStatistics();
            statistics.setVenderId(venderId);
            venderStatisticsMap.put(venderId, statistics);
        }else
            statistics = venderStatisticsMap.get(venderId);
        if(statistics == null)
            return;
        statistics.setContractCount(statistics.getContractCount() + contractCount);
        statistics.setNearlyExpireCount(statistics.getNearlyExpireCount() + nearlyExpireCount);
        statistics.setTotalCount(statistics.getTotalCount() + totalCount);
    }
}
